#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Enumerations.cs
	Copyright  2003 Bill Menees.  All rights reserved.
	Bill@Menees.com

	Who		When		What
	-------	----------	-----------------------------------------------------
	BMenees	5.4.2003	Created.

-----------------------------------------------------------------------------*/

#endregion

namespace Menees
{
    #region Using Directives

    using System;
    using System.IO;

    #endregion

    #region public enum CommandLineParseResult

    /// <summary>
    /// Defines the return states for <see cref="CommandLine.Parse(IEnumerable{string})"/>.
    /// </summary>
    public enum CommandLineParseResult
    {
        /// <summary>
        /// Unknown or unparsed.
        /// </summary>
        Unknown,

        /// <summary>
        /// All arguments were valid.
        /// </summary>
        Valid,

        /// <summary>
        /// One or more arguments were invalid.
        /// </summary>
        Invalid,

        /// <summary>
        /// The user requested help.
        /// </summary>
        HelpRequested,
    }

    #endregion

    #region public enum CommandLineSwitchOptions

    /// <summary>
    /// Defines the options for <see cref="CommandLine"/>'s AddSwitch methods.
    /// </summary>
    [Flags]
    public enum CommandLineSwitchOptions
    {
        /// <summary>
        /// Use the default switch behavior (i.e., optional and single use).
        /// </summary>
        None = 0,

        /// <summary>
        /// The switch is required.
        /// </summary>
        Required = 1,

        /// <summary>
        /// Allow multiple uses of the switch.
        /// </summary>
        AllowMultiple = 2,
    }

    #endregion

    #region public enum CommandLineWriteOptions

    /// <summary>
    /// Defines the options for <see cref="CommandLine.WriteMessage(TextWriter, CommandLineWriteOptions)"/>.
    /// </summary>
    [Flags]
    public enum CommandLineWriteOptions
    {
        /// <summary>
        /// Don't write any output.
        /// </summary>
        None = 0,

        /// <summary>
        /// Write out the header.
        /// </summary>
        Header = 1,

        /// <summary>
        /// Write out the error message.
        /// </summary>
        Error = 2,

        /// <summary>
        /// Write out the help.
        /// </summary>
        Help = 4,
    }

    #endregion

	#region public enum OSStyle

	/// <summary>
	/// The operating systems styles supported by the Menees library.
	/// </summary>
	/// <remarks>
	/// These values are ordered so I can compare them to check OS-levels.  I didn't break
	/// Win9x/Me down because I consider them toy OSes inferior to any NT variation.
	/// </remarks>
	public enum OSStyle 
	{ 
		/// <summary>
		/// Windows 95, 98, or Me.
		/// </summary>
		Win9x = 0,
		/// <summary>
		/// NT 4.0
		/// </summary>
		NT4 = 40, 
		/// <summary>
		/// Windows 2000
		/// </summary>
		Windows2000 = 50, 
		/// <summary>
		/// Windows XP
		/// </summary>
		WindowsXP = 51,
		/// <summary>
		/// Windows Vista
		/// </summary>
		WindowsVista = 60,
		/// <summary>
		/// Windows 7
		/// </summary>
		Windows7 = 61,
	};

	#endregion

	#region public enum SystemSound

	/// <summary>
	/// The supported types of system sounds that can be played by <see cref="Utilities.PlaySound"/>.
	/// </summary>
	public enum SystemSound 
	{ 
		/// <summary>
		/// The default sound.
		/// </summary>
		Default = 0,
		/// <summary>
		/// The question/confirmation sound.
		/// </summary>
		Question = 0x20, 
		/// <summary>
		/// The error sound.
		/// </summary>
		Error = 0x10, 
		/// <summary>
		/// The warning sound.
		/// </summary>
		Warning = 0x30, 
		/// <summary>
		/// The information sound.
		/// </summary>
		Information = 0x40, 
		/// <summary>
		/// A simple sound.
		/// </summary>
		Simple = -1 
	};

	#endregion
}
